/*
 *    ks2tex.c  : a ks-to-tex converter
 *
 *    Revision history:
 *
 *    Created By uhhyung@daiduk.kaist.ac.kr on 10-Oct-1991.
 *	minor modification by yunju@camars.kaist.ac.kr on 16-Oct-1991     
 * 
 *    Comments:
 *
 *    String XCHARS is defined to allow extra space \hX for specific characters
 *    which comes just before a Hangul character.
 *    Macro \hS is for similar cases execpt that it's for English alphabets.
 *
 *    Feel free to customize the values defined in "hfont.tex".
 *	
 *
 *    This is freely distributable under GPL license term.
 *
 */

#include <stdio.h>
#include <ctype.h>
#include "table.h"

#define XCHARS	"$()"

int line;

void usage()
{
	fputs("Usage: htex [Input] [Output]\n",stderr);
}

int Fgetc(in)
FILE *in;
{
	int c;
	c = fgetc(in);
	if(c=='\n') line++;
	return c;
}

void main(argc,argv)
int argc;
char **argv;
{
	FILE *in,*out;
	extern unsigned char htbl[][4];
	int opt, ch=0, last=0, font=0;
	while((opt=getopt(argc,argv,"h"))!=-1) {
			usage();
			exit(0);
	}
	in = argc > 1 ? fopen(argv[1],"r") : stdin;
	out = argc > 2 ? fopen(argv[2],"w") : stdout;
	if(in==NULL){
		printf("%s : not found\n",argv[1]);
		exit(0);
	}
	if(out==NULL){
		printf("%s : not found\n",argv[2]);
		exit(0);
	}
	if (argc==1) {
		puts("% htex : KS hangul to ASCII TeX code converter");
		puts("% Press 'ctrl-d' \n%");
	}
	line=1;
	ch = Fgetc(in);
	while(!feof(in)) {
		if (ch>160) {
			int code,code2;
			code = 94*(ch-176);
			code2 = Fgetc(in);
			if(code2<=160){
				fprintf(stderr,"bad char at line %d\n",line);
				ch = Fgetc(in);
				continue;
			}
			code +=(code2-161);
			if(code<0||code>2349) {
				fprintf(stderr,"bad char at line %d\n",line);
				ch = Fgetc(in);
				continue;
			}
			fputc('{',out);
			if (isalpha(last)) fprintf(out,"\\hS");
 			if (strchr(XCHARS,last)) fprintf(out, "\\hX");

			font = htbl[code][0];
			if (font==1) fprintf(out,"\\hA");
			else fprintf(out,"\\hB");
			if (htbl[code][3]) {
				fprintf(out,"\\hC%d\\hC%d\\hC%d",
				htbl[code][1],htbl[code][2],
				htbl[code][3]);
			} else {
				fprintf(out,"\\hC%d\\hC%d",
				htbl[code][1],htbl[code][2]);
			}
			last = ch;
			ch = Fgetc(in);
			if (ch>160) fprintf(out,"\\hP");
			fputc('}',out);
		} else {
			putc(ch,out);
			last = ch;
			ch = Fgetc(in);
		}
	}
	fclose(in);
	fclose(out);
}
