/*
 *    tex2ks.c  
 *
 */

#include <stdio.h>
#include <ctype.h>

/* TEX to KS Hangul Converter 
 *  
 *  Author : Yunju Baek (yunju@camars.kaist.ac.kr)
 */

Getin(c)
int *c;
{
	scanf("%d",c);
}

Output2(c)
int c;
{
	if(c!=-1) printf("%c%c",(c&0xffff)>>8,c&0xff);
	else fprintf(stderr,"error\n");
}

int line;

void usage()
{
	fprintf(stderr,"Usage: tex2ks [Input] [Output]\n");
}

int Fgetc()
{
	int c;
	c = fgetc(stdin);
	if(c=='\n') line++;
	return c;
}


int Fungetc(c)
int c;
{
	ungetc(c,stdin);
	if(c=='\n') line--;
}

void main(argc,argv)
int argc;
char **argv;
{
	extern unsigned char htbl[][4];
	char t[1024];
	int opt, ch=0,ti=0;
	while((opt=getopt(argc,argv,"h"))!=-1) {
			usage();
			exit(1);
	}
	switch (argc)
	  {
	    case 1:
	      break;
	    case 2:
	      if (freopen(argv[1], "r", stdin) == NULL)
		{
		    fprintf(stderr, "Could not open %s\n", argv[1]);
		    exit(1);
		}
	      break;
	    case 3:
	      if (freopen(argv[1], "r", stdin) == NULL)
		{
		    fprintf(stderr, "Could not open %s\n", argv[1]);
		    exit(1);
		}
	      if (freopen(argv[2], "w+", stdout) == NULL)
		{
		    fprintf(stderr, "Could not open %s\n", argv[2]);
		    exit(1);
		}
	      break;
	    default:
	      usage();
	      exit(1);
	  }

	t[0] = '\0'; ti = 0;
	while (( ch = Fgetc()) != EOF){
	    if(ch == ' '|| ch == '\n' || 
	       ch == '\t' || ch == '\r'){
		printf("%s%c",t,ch); t[0] = '\0'; ti = 0;
	    }
	    else {
		if(ch == '\{'){
		    if(ti>0) printf("%s",t);
		    t[0] = ch; ti = 1;
		}else if(ch == 0200){ /* newfont scheme*/
		    if(ti>0) printf("%s",t);
		    t[0] = '\0'; ti = 0;
		    parse_ks2();
		}else{
		    t[ti++] = ch;
		}
		t[ti] = '\0';
		if(strlast(t,"\\hA",3)){
		    parse_ks(1);
		    t[0]='\0'; ti = 0;
		}else if(strlast(t,"\\hB",3)){
		    parse_ks(2);
		    t[0]='\0'; ti = 0;
		}
	    }
	}
	fclose(stdin);
	fclose(stdout);
}

parse_ks(hc)
int hc;
{
    char t[1024];
    int ti=0;
    int ch,val,time=0,code=0,flag=0;
    

	t[0] = '\0'; ti = 0;
	while (( ch = Fgetc()) != EOF){
	    if(ch == '\}') return;
	    else if(ch == ' '|| ch == '\n' || 
	       ch == '\t' || ch == '\r'){
		parse_end(); return;
	    }
	    t[ti++] = ch; t[ti] = '\0';
	    if(!strncmp("\\hC",t,3)){
		time++;
		t[0] = '\0'; ti = 0;
		while (( t[ti] = Fgetc()) != EOF){
		    if( t[ti] == '\\' || t[ti] == '\}'){
			Fungetc(t[ti]);
			t[ti] = '\0';
			val = atoi(t);
			flag = e2ks(hc,val,time,&code);
			t[0]='\0'; ti=0;
			if(flag==0){
			    while (( ch = Fgetc()) != EOF && ch != '\}') ;
			    Output2(code);
			    return;
			}
			else if(flag == -1){
			    parse_end();
			    return;
			}
			else break;
		    }
		    else ti++;
		}
		      
	    }else if(!strncmp("\\hX",t,3) || !strncmp("\\hP",t,3) || 
		    !strncmp("\\hS",t,3)) {
		t[0] = '\0'; ti = 0;
		continue; 
	    }else if(ti>=4){
		parse_end();
		return;
	    }
	}

}

int h2d( c)
    int c;
{
    if(c >= 060 && c <= 071)
      return c - 060;
    if(c >= 0101 && c <= 0106)
      return c - 0101 + 10;
    else 
      return -1;
}
parse_ks2()
{
    int hc,ch,ch2,val,time=0,code=0,flag=0,i=0,n;
    
    if(( ch = Fgetc()) != EOF){
	switch(ch){
	  case 0201:	
	  case 0211:	
	    hc = 1;		n = 1; 	break;
	  case 0202:	
	  case 0212:	
	    hc = 1;		n = 2; 	break;
	  case 0203:	
	  case 0213:	
	    hc = 1;		n = 3; 	break;
	  case 0204:	
	  case 0214:	
	    hc = 2;		n = 1; 	break;
	  case 0205:	
	  case 0215:	
	    hc = 2;		n = 2; 	break;
	  case 0206:	
	  case 0216:	
	    hc = 2;		n = 3; 	break;
	  case 0210:	
	    return;
	  default:
	    parse_end2();
	    return;
	}
    }
    else{
	parse_end2();
	return;
    }

    while((( ch = Fgetc()) != EOF)&&ch != 0207&& i < n){
	if(ch=='.') continue;
	if((( ch2 = Fgetc()) != EOF)&&ch != 0207){
	    i++;
	    if(ch2=='.') val = h2d(ch);
	    else val =  h2d(ch)*16 + h2d(ch2);
	    flag = e2ks(hc,val,i,&code);
	    if(flag==0){
		while (( ch = Fgetc()) != EOF && ch != 0207) ;
		Output2(code);
		return;
	    }
	    else if(flag == -1){
		parse_end2();
		return;
	    }
	}
    }
}

parse_end()
  {
      int ch;
      printf(stderr,"Fail to guess at line %d\n",line);
      while (( ch = Fgetc()) != EOF && ch != '\}') ;
  }

parse_end2()
  {
      int ch;
      printf(stderr,"Fail to guess at line %d\n",line);
      while (( ch = Fgetc()) != EOF && ch != 0207) ;
  }

strlast(s1,s2,n)
char *s1,*s2;
int n;
{
    char *a,*b;
    int i=0;

    a = s1; b = s2;

    while(*a){ a++; i++; }
    if(i<n) return 0;
    a--;

    i=0;
    while(*b){ b++; i++; }
    if(i<n) return 0;
    b--;
    
    for(i=0;i<n;i++){
	if(*a-- != *b--)
	  return 0;
    }
    return 1;
}

